/* PIO/PEX-D48 Linux Static Libary.

   Author: Winson Chen

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/* File level history (record changes for this file here.)

   Use vi or gvim to edit this file and set tabstop=4.

   v 0.0.0  25 Mar 2019 by Winson Chen
     function created for digital I/O and signal interrupt.

*/

#include "piodio.h"
#include "inter.h"

boolean port_mapping(int port);
unsigned char port_status_82551 = 155, port_status_82552 = 155;
int P2H = 1, P2L = 1, P5H = 1, P5L = 1;	//port2/5 high/low nibble default is DI

boolean port_mapping(int port)
{
	int id;

	switch(port)
	{
		case PIOD48_P0:
			id = IXPIO_82551PA;
		break;
		case PIOD48_P1:
			id = IXPIO_82551PB;
		break;
		case PIOD48_P2:
			id = IXPIO_82551PC;
		break;
		case PIOD48_P2_HB:
			id = IXPIO_82551PC;
		break;
		case PIOD48_P2_LB:
			id = IXPIO_82551PC;
		break;
		case PIOD48_P3:
			id = IXPIO_82552PA;
		break;
		case PIOD48_P4:
			id = IXPIO_82552PB;
		break;
		case PIOD48_P5:
			id = IXPIO_82552PC;
		break;
		case PIOD48_P5_HB:
			id = IXPIO_82552PC;
		break;
		case PIOD48_P5_LB:
			id = IXPIO_82552PC;
		break;
		default:
			id = -1;
		break;
	}

	return id;
}


//1 digital input, 0 digital output
/* 8255-1 config port0~port2 (0xcc)   piod48&pexd48 CN1 PA0~7 PB0~7 PC0~7  piod48su CN1 PA00~07 PB00~07 PC00~07
   8255-2 config port3~port5 (0xdc)   piod48&pexd48 CN2 PA0~7 PB0~7 PC0~7  piod48su CN1 PA10~17 PB10~17 PC10~17*/
WORD PIOD48_PortDirCfs(WORD fd, WORD port, boolean io)
{
	ixpio_reg_t reg;

	switch(port)
	{
		case PIOD48_P0:	//CN1 PA, port0
		{
			reg.id = IXPIO_82551CW;
			if(io)
				port_status_82551 |= 0x10;
			else
				port_status_82551 &= 0xef;
		}
		break;
		case PIOD48_P1:	//CN1 PB, port1
		{
			reg.id = IXPIO_82551CW;
			if(io)
				port_status_82551 |= 0x2;
			else
				port_status_82551 &= 0xfd;
		}
		break;
		case PIOD48_P2:	//CN1 PC, port2
		{
			reg.id = IXPIO_82551CW;
			if(io)
			{
				port_status_82551 |= 0x9;
				P2H = 1;
				P2L = 1;
			}
			else
			{
				port_status_82551 &= 0xf6;
				P2L = 0;
				P2L = 0;
			}
		}
		break;
		case PIOD48_P2_HB:	//CN1 PC_HB, port2
		{
			reg.id = IXPIO_82551CW;
			if(io)
			{
				port_status_82551 |= 0x8;
				P2H = 1;
			}
			else
			{
				port_status_82551 &= 0xf7;
				P2H = 0;
			}
		}
		break;
		case PIOD48_P2_LB:	//CN1 PC_LB, port2
		{
			reg.id = IXPIO_82551CW;
			if(io)
			{
				port_status_82551 |= 0x1;
				P2L = 1;
			}
			else
			{
				port_status_82551 &= 0xfe;
				P2L = 0;
			}
		}
		break;
		case PIOD48_P3:	//CN2 PA, port3
		{
			reg.id = IXPIO_82552CW;
			if(io)
				port_status_82552 |= 0x10;
			else
				port_status_82552 &= 0xef;
		}
		break;
		case PIOD48_P4:	//CN2 PB, port4
		{
			reg.id = IXPIO_82552CW;
			if(io)
				port_status_82552 |= 0x2;
			else
				port_status_82552 &= 0xfd;
		}
		break;
		case PIOD48_P5:	//CN2 PC, port5
		{
			reg.id = IXPIO_82552CW;
			if(io)
			{
				port_status_82552 |= 0x9;
				P5H = 1;
				P5L = 1;
			}
			else
			{
				port_status_82552 &= 0xf6;
				P5H = 0;
				P5L = 0;
			}
		}
		break;
		case PIOD48_P5_HB:	//CN2 PC_HB, port5
		{
			reg.id = IXPIO_82552CW;
			if(io)
			{
				port_status_82552 |= 0x8;
				P5H = 1;
			}
			else
			{
				port_status_82552 &= 0xf7;
				P5H = 0;
			}
		}
		break;
		case PIOD48_P5_LB:	//CN2 PC_LB, port5
		{
			reg.id = IXPIO_82552CW;
			if(io)
			{
				port_status_82552 |= 0x1;
				P5L = 1;
			}
			else
			{
				port_status_82552 &= 0xfe;
				P5L = 0;
			}
		}
		break;
	}

	if(port <= IXPIO_PORT_2_LB)
		reg.value = port_status_82551;
	else
		reg.value = port_status_82552;

	// setup I/O Selection Control Register
	if (PIODA_OutputByte(fd, &reg)) return FAILURE;

	return SUCCESS;
}

WORD PIOD48_Digital_Output(WORD fd, WORD port, byte data)
{
	ixpio_reg_t reg;
	int id;
	
	id = port_mapping(port);
	if(id < 0)
	{
		//printf("Port define error!\n");
		return PIODA_PORT_DEFINED_ERROR;
	}

	reg.id = id;
	reg.value = data;

	// setup I/O Selection Control Register
	if (PIODA_OutputByte(fd, &reg)) return PIODA_DIGITAL_OUTPUT_ERROR;

	return SUCCESS;
}

WORD PIOD48_Digital_Input(WORD fd, WORD port, WORD *di_data)
{
	ixpio_reg_t reg;
	int id;

	id = port_mapping(port);
	if(id < 0)
	{
		//printf("Port define error!\n");
		return PIODA_PORT_DEFINED_ERROR;
	}

	reg.id = id;

	if (PIODA_InputByte(fd, &reg)) return PIODA_DIGITAL_INPUT_ERROR;
  
	*di_data = reg.value;
  
	if(port == PIOD48_P2)
	{
		//port2 high nibble set as DI port, 
		if(!P2H)
			*di_data = *di_data & 0xf;
		if(!P2L)
			*di_data = *di_data & 0xf0;
	}

	if(port == PIOD48_P5)
	{
		//port5 high nibble set as DI port, 
		if(!P5H)
			*di_data = *di_data & 0xf;
		if(!P5L)
			*di_data = *di_data & 0xf0;
	}
	return SUCCESS;
}

WORD PIOD48_Int_Source_Init(WORD fd, WORD signal, WORD int_source)
{
	ixpio_reg_t reg;

	if(int_source < 0 || int_source > 0xf)
		return PIODA_INT_SOURCE_DEFINED_ERROR;

	// configure board interrupt
	reg.id = IXPIO_IMCR;	//0x05
	reg.value = int_source;

	if(PIODA_OutputByte(fd, &reg))
	{
		PIODA_IntRemove(fd, signal);
		return PIODA_CONFIGURE_INTERRUPT_ERROR;
	}

	return PIODA_NOERROR;
}

/*
	Int 0  INT_CHAN_0	option	0  PC3&!PC7
	    1  INT_CHAN_1		1  disable Int
					2  PC3
*/
WORD PIOD48_INT01_CONTROL(WORD fd, WORD Int, WORD opt)
{
	ixpio_reg_t reg;
	int tmp = 0;

	if(Int > 1 || Int < 0 || opt > 2 || opt < 0)
		return PIODA_LIBRARY_PARAMETER_ERROR;

	if(Int)	//INT_CHAN_1
		opt = (opt << 4) | tmp;
	else	//INT_CHAN_0
		opt = (opt << 2) | tmp;

		reg.id = IXPIO_CICR;	//0xf0
		reg.value = opt;
		printf(" ");
		if (PIODA_OutputByte(fd, &reg)) return PIODA_DIGITAL_OUTPUT_ERROR;

	return PIODA_NOERROR;
}

WORD PIOD48_8254Clk(WORD fd, WORD boolean)       //winson
{
	ixpio_reg_t reg;

	reg.id = IXPIO_CICR;    //0xf0
	reg.value = 0x14 | boolean;

	if (ioctl(fd, IXPIO_WRITE_REG, &reg)) {
		close(fd);
		puts("Failure of choosing clock.");
		return FAILURE;
	}

	return PIODA_NOERROR;
}

WORD PIOD48_8254CW(WORD fd, WORD bcd, WORD mode, WORD rl, WORD sc)
{
	ixpio_reg_t reg;
	static struct sigaction act_old;

	reg.id = IXPIO_8254CW;  //0xec
	reg.value = bcd + (mode << 1) + (rl << 4) + (sc << 6);

	if (ioctl(fd, IXPIO_WRITE_REG, &reg)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of 8254 counter word.");
		return FAILURE;
	}

	return PIODA_NOERROR;
}

WORD PIOD48_8254C0(WORD fd, WORD hbyte, WORD lbyte)
{
	ixpio_reg_t reg;
	static struct sigaction act_old;

	reg.id = IXPIO_8254C0;			/* Notice Counter1 can't be 1 */
	reg.value = lbyte;			/* Set 8254 counter 0 LSB */
	if (ioctl(fd, IXPIO_WRITE_REG, &reg)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of 8254 counter 0 LSB.");
		return FAILURE;
	}

	reg.value = hbyte;			/* Set 8254 counter 0 MSB */
	if (ioctl(fd, IXPIO_WRITE_REG, &reg)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of 8254 counter 0 MSB.");
		return FAILURE;
	}

	return PIODA_NOERROR;
}

WORD PIOD48_8254C1(WORD fd, WORD hbyte, WORD lbyte)
{
	ixpio_reg_t reg;
	static struct sigaction act_old;

	reg.id = IXPIO_8254C1;			/* Notice Counter1 can't be 1 */
	reg.value = lbyte;			/* Set 8254 counter 1 LSB */
	if (ioctl(fd, IXPIO_WRITE_REG, &reg)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of 8254 counter 1 LSB.");
		return FAILURE;
	}

	reg.value = hbyte;			/* Set 8254 counter 1 MSB */
	if (ioctl(fd, IXPIO_WRITE_REG, &reg)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of 8254 counter 1 MSB.");
		return FAILURE;
	}

	return PIODA_NOERROR;
}

WORD PIOD48_8254C2(WORD fd, WORD hbyte, WORD lbyte)
{
	ixpio_reg_t reg;
	static struct sigaction act_old;

	reg.id = IXPIO_8254C2;			/* Notice Counter1 can't be 1 */
	reg.value = lbyte;			/* Set 8254 counter 2 LSB */
	if (ioctl(fd, IXPIO_WRITE_REG, &reg)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of 8254 counter 2 LSB.");
		return FAILURE;
	}

	reg.value = hbyte;			/* Set 8254 counter 2 MSB */
	if (ioctl(fd, IXPIO_WRITE_REG, &reg)) {
		close(fd);
		sigaction(SIGALRM, &act_old, NULL);
		puts("Failure of 8254 counter 2 MSB.");
		return FAILURE;
	}

	return PIODA_NOERROR;
}
