/* PIO/PISO Linux Static Library.

   Author: Golden Wang

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */

/* File level history (record changes for this file here.)

   Use vi or gvim to edit this file and set tabstop=4.
   v 0.2.3  16  Aug 2019 by Winson Chen
     give support to piso/pex-p16r16

   v 0.2.2  10  Jul 2019 by Winson Chen
     give support to pio/pex-d48

   v 0.2.1  25  Aug 2010 by Golden Wang
     fixed bug that get the name of device module.
     fixed function "PIODA_GetDriverVersion" and "PIODA_GetLibraryVersion".
     fixed function's return value.

   v 0.2.0  7  Sep 2007 by Golden Wang
     give support to pisoa64/c64.
     give support to pisop32a32/p32c32.
     give support to pisop64.

   v 0.1.2  7  Sep 2007 by Golden Wang
     give support to piod96.

   v 0.1.1  7  Jul 2007 by Golden Wang
     give support to piod24/d56.

   v 0.1.0  11 Jun 2007 by Golde Wang
     give support to piod144/168.  

   v 0.0.0  24 May 2007 by Golden Wang
     create, blah blah.
    	
*/

/* the header that defined all variable and funtions */ 
#include "piodio.h"
#include "inter.h"

/* declared private function here */
WORD PIODA_Init(WORD);
WORD PIODA_GetDeviceModuleName(int);
WORD PIODA_Set_Device_Module(char dev_file[], int);
WORD PIODA_ReadEEPROMAll(WORD);
WORD PIODA_EEP_READ(WORD, WORD, WORD *, WORD *);
WORD PIODA_COMM(WORD);
WORD PIODA_WR_BYTE(WORD, WORD);
WORD PIODA_RD_BYTE(WORD, WORD *);
int find_card_num(int);

//port_config_t *pconf;
//dev_control_t *dev_ctl;
dev_control_t card_record[16] = {[0 ... 15].fd = -1};	//record each cards by fd, and maximum cards is 16 in this lib.
 
static float PIODA_fDeltaV[MAX_BOARD_NUMBER][16];
static float PIODA_fDeltaI[MAX_BOARD_NUMBER][16];
static WORD  PIODA_wN10V[MAX_BOARD_NUMBER][16];
static WORD  PIODA_wP10V[MAX_BOARD_NUMBER][16];
static WORD  PIODA_w00mA[MAX_BOARD_NUMBER][16];
static WORD  PIODA_w20mA[MAX_BOARD_NUMBER][16];
static WORD  PIODA_EEP;

//char device_module[256];
char ixpio[MAX_CARD_SUPPORT][LINE_SIZE];
WORD channel_record;
WORD channel_init = 1;
WORD gain_record;
WORD gain_init = 1;

dev_control_t* find_fd(int fd)
{
	int i;
	dev_control_t *dev_ctl;

	for(i = 0; i < 16; i++)
	{
		if(fd == card_record[i].fd)
		{
			dev_ctl = &card_record[i];
			break;
		}
	}

	return dev_ctl;
}

int find_card_num(int fd)
{
	int i;
	for(i = 0; i < 16; i++)
        {
                if(fd == card_record[i].fd)
			return i;
        }

}

/* open ixpio device file */
int PIODA_Open(char *dev_file)
{
	int fd, i, card_num;
	char t = 'o';
	char *str;

	fd = open(dev_file, O_RDWR);

	str = strchr(dev_file, t);
	card_num = atoi(++str);

	card_record[card_num].fd = fd;

	return fd;
}

/* close ixpio device file */
WORD PIODA_Close(WORD fd)
{
	/* free malloc memory */
//	if (dev_ctl) 
//		free(dev_ctl);

	/* close device file */
	close(fd);

	return PIODA_NOERROR;
}

WORD PIODA_DriverInit(WORD fd)
{
	dev_control_t *dev_ctl;

	dev_ctl = find_fd(fd);

	if(PIODA_GetDeviceModuleName(fd)) 
		return PIODA_MODULE_NAME_GET_ERROR;

	/* get EEPROM data */
	if (!strcmp(dev_ctl->device_module,"ixpioda16"))
	{
		if(PIODA_ReadEEPROMAll(fd)) 
			return PIODA_READ_EEPROM_ERROR;
	}

	PIODA_Init(fd);

	return PIODA_NOERROR;
}

WORD PIODA_DeviceInit(WORD fd)
{
        WORD rs;
	dev_control_t *dev_ctl;
	int port_num = 0;

	dev_ctl = find_fd(fd);

        /* port status are changed */
        if (!strcmp(dev_ctl->device_module,"ixpiod144"))
        {
		port_num = 17;		//0~17
        }
        else if (!strcmp(dev_ctl->device_module,"ixpiod168"))
        {
		port_num = 20;		//0~20
        }
/*
        else if (!strcmp(dev_ctl->device_module,"ixpiod96"))
        {
                if ( (short)port < 0 || (short)port > 11 )
                        return PIODA_PORT_DEFINED_ERROR;
        }
        else if (!strcmp(dev_ctl->device_module,"ixpiod56") || !strcmp(dev_ctl->device_module,"ixpiod24"))
        {
                if ( (short)port < 0 || (short)port > 2 )
                        return PIODA_PORT_DEFINED_ERROR;
        }
*/
        rs = dev_ctl->deviceinit(fd, port_num);
        if (rs)
                return PIODA_DEVICE_DIO_INIT_ERROR;

        return PIODA_NOERROR;;
}

/* get ixpio device driver version that be defined in piodio.h */ 
char* PIODA_GetDriverVersion(void)
{
	//printf("Drive VERSION %s\n",IXPIO_DRIVER_VERSION);
	return IXPIO_DRIVER_VERSION;
}

char* PIODA_GetLibraryVersion(void)
{
	//printf("Library VERSION %s\n",IXPIO_LIBRARY_VERSION);
	return IXPIO_LIBRARY_VERSION;
}

/* change port status */
WORD PIODA_PortDirCfs(WORD fd, WORD port, boolean io)
{
	WORD rs;
	dev_control_t *dev_ctl;
	dev_ctl = find_fd(fd);

	/* port status are changed */   
	if (!strcmp(dev_ctl->device_module,"ixpiod144"))
	{
		if ( (short)port < 0 || (short)port > 17 ) 
			return PIODA_PORT_DEFINED_ERROR;
	}
	else if (!strcmp(dev_ctl->device_module,"ixpiod168"))
	{
		if ( (short)port < 0 || (short)port > 20 ) 
			return PIODA_PORT_DEFINED_ERROR;
	}
	else if (!strcmp(dev_ctl->device_module,"ixpiod96"))
	{
		if ( (short)port < 0 || (short)port > 11 ) 
			return PIODA_PORT_DEFINED_ERROR;
	} 
	else if (!strcmp(dev_ctl->device_module,"ixpiod56") || !strcmp(dev_ctl->device_module,"ixpiod24"))
	{
		if ( (short)port < 0 || (short)port > 2 ) 
			return PIODA_PORT_DEFINED_ERROR;
	}
	else if (!strcmp(dev_ctl->device_module,"ixpiod48_pexd48"))
	{
		if(port == PIOD48_P0 || port == PIOD48_P1 || port == PIOD48_P2 || port == PIOD48_P2_HB || port == PIOD48_P2_LB || 
		   port == PIOD48_P3 || port == PIOD48_P4 || port == PIOD48_P5 || port == PIOD48_P5_HB || port == PIOD48_P5_LB)
			;
		else
			return PIODA_PORT_DEFINED_ERROR;
	}

	rs = dev_ctl->portdircfs(fd, port, io);

	if (rs)
		return PIODA_DEVICE_DIO_INIT_ERROR;

	return PIODA_NOERROR;;
}

/* output digital data */
WORD PIODA_Digital_Output(WORD fd, WORD port, WORD data)
{
	WORD rs;
	dev_control_t *dev_ctl;

	dev_ctl = find_fd(fd);

	if (!strcmp(dev_ctl->device_module,"ixpiod144") || !strcmp(dev_ctl->device_module,"ixpiod168") || 
            !strcmp(dev_ctl->device_module,"ixpiod96") || !strcmp(dev_ctl->device_module,"ixpiod56") || 
            !strcmp(dev_ctl->device_module,"ixpiod24") || !strcmp(dev_ctl->device_module,"ixpiod48_pexd48") )
	{
		rs = dev_ctl->doutput_port(fd, port, data);
	}
	else
	{
		if ( !strcmp(dev_ctl->device_module,"ixpisoa64") || 
                     !strcmp(dev_ctl->device_module,"ixpisop32c32") || 
                     !strcmp(dev_ctl->device_module,"ixpisop32a32") ||
		     !strcmp(dev_ctl->device_module,"ixpisop16r16u"))
		{
			rs = dev_ctl->doutput_port(fd, port, data);
		}
		else	//PISO-P8R8, PISO-725, PIO-DA16
		{
			rs = dev_ctl->doutput(fd, data);
		}
	}

	if(rs)
		return rs;

	return PIODA_NOERROR;
}

/* input digital data */
WORD PIODA_Digital_Input(WORD fd, WORD port, WORD *di_data)
{
	WORD rs;
	dev_control_t *dev_ctl;

	dev_ctl = find_fd(fd);

	if (!strcmp(dev_ctl->device_module,"ixpiod144") || !strcmp(dev_ctl->device_module,"ixpiod168") || 
            !strcmp(dev_ctl->device_module,"ixpiod96") || !strcmp(dev_ctl->device_module,"ixpiod56") || 
            !strcmp(dev_ctl->device_module,"ixpiod24") || !strcmp(dev_ctl->device_module,"ixpiod48_pexd48") )
	{
		rs = dev_ctl->dinput_port(fd, port, di_data);
	}
	else
	{
		if (!strcmp(dev_ctl->device_module,"ixpisop64") || 
                    !strcmp(dev_ctl->device_module,"ixpisop32c32") || 
                    !strcmp(dev_ctl->device_module,"ixpisop32a32") ||
		    !strcmp(dev_ctl->device_module,"ixpisop16r16u"))
		{
			rs = dev_ctl->dinput_port(fd, port, di_data);
		}
		else	//PISO-P8R8, PISO-725, PIO-DA16
		{
			rs = dev_ctl->dinput(fd, di_data);
		}
	}

	if(rs)
		return rs;

	return PIODA_NOERROR;
}


// ******************************************************************
// Analog-Output for Voltage with calibration
//
// Parameters :
//   wBoardNo  :[in] Board Number, Range is 0 to 7.
//   wChannels :[in] Analog-Output to which channel ?
//                   0 to 15 for PIO-DA16
//                   0 to  7 for PIO-DA8
//                   0 to  3 for PIO-DA4
//   fvalue    :[in] -10v to +10v
// ******************************************************************
WORD PIODA_AnalogOutputCalVoltage(WORD fd, WORD channel, float fvalue)
{
	WORD data;
	WORD wBase;
	float fTmp;

	fTmp = PIODA_fDeltaV[fd][channel];

	if ( fTmp != 0 )
	{
		data = (WORD)( (fvalue+10.0F)/fTmp  + PIODA_wN10V[fd][channel] );
	} 
	else
	{
		return PIODA_EEPROM_DATA_ERROR;
	}

	ixpio_analog_t ao;

	ao.channel = channel;  
	ao.data.u16 = data;

	if (PIODA_OutputAnalog(fd, &ao)) 
		return PIODA_OUTPUT_CALVOLTAGE_ERROR;
  
	return PIODA_NOERROR;
}

// ******************************************************************
// Analog-Output for Voltage without calibration
//
// Parameters :
//   wBoardNo  :[in] Board Number, Range is 0 to 7.
//   wChannels :[in] Analog-Output to which channel ?
//                   0 to 15 for PIO-DA16
//                   0 to  7 for PIO-DA8
//                   0 to  3 for PIO-DA4
//   fvalue    :[in] -10v to +10v
// ******************************************************************
WORD PIODA_AnalogOutputVoltage(WORD fd, WORD channel, float fvalue)
{
	WORD data;
	float fStep;

	fStep=16383.0F/20.0F;

	data= (WORD)(fvalue*fStep+8192.0F);

	ixpio_analog_t ao;
	ao.channel = channel;
	ao.data.u16 = data;

	if (PIODA_OutputAnalog(fd, &ao)) 
		return PIODA_OUTPUT_VOLTAGE_ERROR;  

	return PIODA_NOERROR;
}

// ******************************************************************
// Analog-Output for Current with calibration
//
// Parameters :
//   wBoardNo  :[in] Board Number, Range is 0 to 7.
//   wChannels :[in] Analog-Output to which channel ?
//                   0 to 15 for PIO-DA16
//                   0 to  7 for PIO-DA8
//                   0 to  3 for PIO-DA4
//   fvalue    :[in] 0mA to +20mA
// ******************************************************************
WORD PIODA_AnalogOutputCalCurrent(WORD fd, WORD channel, float fvalue)
{
	WORD data;
	float fTmp;
	ixpio_analog_t ao;

	fTmp = PIODA_fDeltaI[fd][channel];

	if ( fTmp != 0 )
	{
		data = (WORD)( fvalue/fTmp  + PIODA_w00mA[fd][channel] );
	}
	else
	{
		return PIODA_EEPROM_DATA_ERROR;
	}

	ao.channel = channel; 
	ao.data.u16 = data;

	if (PIODA_OutputAnalog(fd, &ao)) 
		return PIODA_OUTPUT_CALCURRENT_ERROR;
  
	return PIODA_NOERROR;
}

// ******************************************************************
// Analog-Output for Current without calibration
//
// Parameters :
//   wBoardNo  :[in] Board Number, Range is 0 to 7.
//   wChannels :[in] Analog-Output to which channel ?
//                   0 to 15 for PIO-DA16
//                   0 to  7 for PIO-DA8
//                   0 to  3 for PIO-DA4
//   fvalue    :[in] 0mA to +20mA
// ******************************************************************
WORD PIODA_AnalogOutputCurrent(WORD fd, WORD channel, float fvalue)
{
	WORD data;
	float fStep;
	ixpio_analog_t ao;

	fStep=8192.0F/20.0F;

	data= (WORD)(fvalue*fStep+8191.0F);

	ao.channel = channel;
	ao.data.u16 = data;

	if (PIODA_OutputAnalog(fd, &ao)) return PIODA_OUTPUT_CURRENT_ERROR;

	return PIODA_NOERROR;
}

WORD PIODA_AnalogInputVoltage(WORD fd, WORD channel, WORD gain, float *fvalue)
{
	WORD rs;
	dev_control_t *dev_ctl;

	dev_ctl = find_fd(fd);
 
	rs = dev_ctl->ainput(fd, channel, gain, fvalue);

	return rs;
}

/* support some ixpio card interrupt function */
WORD PIODA_IntInstall(WORD fd, HANDLE hisr, WORD signal, WORD int_source, WORD activemode)
{
	WORD rs;
	dev_control_t *dev_ctl;
	ixpio_signal_t sig;
	struct sigaction act,act_old;

	dev_ctl = find_fd(fd);

	memset(&act, 0, sizeof act);
	act.sa_handler = hisr;
	sigemptyset(&act.sa_mask);
	sigaddset(&act.sa_mask, signal);

	if(sigaction(signal, &act, &act_old))
	{
		close(fd);
		return PIODA_CONFIGURE_INTERRUPT_ERROR;
	}

	/* init I/O card INT and check int_source is right or not */
	rs = dev_ctl->int_source_init(fd, signal, int_source);

	if (rs) 
		return rs;	/* Int init error */

	/* signal condiction */
	sig.sid = signal;
	sig.pid = getpid();
	sig.is = int_source;            /* signaling for the channels */
 
	if (activemode == 2 )
	{
		sig.bedge = int_source;             /* signaling for both edges */
	}
	else if (activemode == 1)
	{
		sig.edge = int_source;        /* signaling for positive edges */
		sig.bedge = 0;                /* don't signal for both edges */
	}
	else if (activemode == 0)
	{
		sig.edge = 0;        /* signaling for negative edges */
		sig.bedge = 0;
	}
	else
	{
		close(fd);
		sigaction(signal, &act_old, NULL);
		return PIODA_ACTIVEMODE_DEFINED_ERROR;
	}

	if (ioctl(fd, IXPIO_SIG, &sig))
	{
		close(fd); 
		sigaction(signal, &act_old, NULL);
		return PIODA_ADD_SIGNAL_ERROR;
	}

	return PIODA_NOERROR;
}

/* close interrupt function */
WORD PIODA_IntRemove(WORD fd, WORD sig_id)
{
	ixpio_reg_t reg;
	struct sigaction act_old;
  
	/* disable all board interrupt */
	reg.id = IXPIO_IMCR;
	reg.value = 0;

	if (PIODA_OutputByte(fd, &reg)){ 
		sigaction(sig_id, &act_old, NULL);
		return PIODA_CONFIGURE_INTERRUPT_ERROR;
	}

	sigaction(sig_id, &act_old, NULL);

	return PIODA_NOERROR;
}

WORD PIODA_OutputAnalog(WORD fd, ixpio_analog_t *p_ao)
{
	if (ioctl(fd, IXPIO_ANALOG_OUT, p_ao))
	{
		close(fd);
		return FAILURE;
	}

	return PIODA_NOERROR;
}

WORD PIODA_OutputByte(WORD fd, ixpio_reg_t *p_reg)
{
	if (ioctl(fd, IXPIO_WRITE_REG, p_reg))
	{
		close(fd);
		return FAILURE;
	}

	return PIODA_NOERROR;
}

WORD PIODA_InputByte(WORD fd, ixpio_reg_t *p_reg)
{
	if (ioctl(fd, IXPIO_READ_REG, p_reg))
	{
		close(fd);
		return FAILURE;
	}

	return PIODA_NOERROR;
}

/*********************************/
/* defined private function here */
/*********************************/

/* init I/O card function */
WORD PIODA_Init(WORD fd)
{
	dev_control_t *dev_ctl;

	dev_ctl = find_fd(fd);
        //dev_ctl = malloc(sizeof(dev_control_t));
        //memset(dev_ctl, 0, sizeof(dev_control_t));

        if (!strcmp(dev_ctl->device_module,"ixpiod144") || !strcmp(dev_ctl->device_module,"ixpiod168"))
        {
                dev_ctl->deviceinit = PIOD144_168_DeviceInit;
                dev_ctl->portdircfs = PIOD144_168_PortDirCfs;
                dev_ctl->doutput_port = PIOD144_168_Digital_Output;
                dev_ctl->dinput_port = PIOD144_168_Digital_Input;
                dev_ctl->int_source_init = PIOD144_168_Int_Source_Init;
        }
        else if (!strcmp(dev_ctl->device_module,"ixpiod56") || !strcmp(dev_ctl->device_module,"ixpiod24"))
        {
                dev_ctl->portdircfs = PIOD56_24_PortDirCfs;
                dev_ctl->doutput_port = PIOD56_24_Digital_Output;
                dev_ctl->dinput_port = PIOD56_24_Digital_Input;
                dev_ctl->int_source_init = PIOD56_24_Int_Source_Init;
        }
	else if (!strcmp(dev_ctl->device_module,"ixpiod48_pexd48"))
	{
		dev_ctl->portdircfs = PIOD48_PortDirCfs;
		dev_ctl->doutput_port = PIOD48_Digital_Output;
		dev_ctl->dinput_port = PIOD48_Digital_Input;
                dev_ctl->int_source_init = PIOD48_Int_Source_Init;
		dev_ctl->C8254W = PIOD48_8254CW;
		dev_ctl->C8254_0 = PIOD48_8254C0;
		dev_ctl->C8254_1 = PIOD48_8254C1;
		dev_ctl->C8254_2 = PIOD48_8254C2;
	}
        else if (!strcmp(dev_ctl->device_module,"ixpiod96"))
        {
                dev_ctl->portdircfs = PIOD96_PortDirCfs;
                dev_ctl->doutput_port = PIOD96_Digital_Output;
                dev_ctl->dinput_port = PIOD96_Digital_Input;
                dev_ctl->int_source_init = PIOD96_Int_Source_Init;
        }
	else if (!strcmp(dev_ctl->device_module,"ixpisop8r8"))
        {
                dev_ctl->doutput = PISOP8R8_Digital_Output;
                dev_ctl->dinput = PISOP8R8_Digital_Input;
        }
        else if (!strcmp(dev_ctl->device_module,"ixpiso725"))
        {
                dev_ctl->doutput = PISO725_Digital_Output;
                dev_ctl->dinput = PISO725_Digital_Input;
                dev_ctl->int_source_init = PISO725_Int_Source_Init;
        }
        else if (!strcmp(dev_ctl->device_module,"ixpioda16"))
        {
                dev_ctl->doutput = PIODA16_Digital_Output;
                dev_ctl->dinput = PIODA16_Digital_Input;
                dev_ctl->int_source_init = PIODA16_Int_Source_Init;
        }
        else if (!strcmp(dev_ctl->device_module,"ixpisoa64") || !strcmp(dev_ctl->device_module,"ixpisoc64"))
        {
                dev_ctl->doutput_port = PISOA64_C64_Digital_Output;
        }
        else if (!strcmp(dev_ctl->device_module,"ixpisop64"))
        {
                dev_ctl->dinput_port = PISOP64_Digital_Input;
        }
        else if (!strcmp(dev_ctl->device_module,"ixpiso813"))
        {
                dev_ctl->ainput = PISO813_Analog_Input_Voltage;
        }
	else if (!strcmp(dev_ctl->device_module,"ixpisop32c32") || !strcmp(dev_ctl->device_module,"ixpisop32a32"))
        {
                dev_ctl->doutput_port = PISOP32A32_C32_Digital_Output;
                dev_ctl->dinput_port = PISOP32A32_C32_Digital_Input;
        }
	else if(!strcmp(dev_ctl->device_module,"ixpisop16r16u"))
	{
                dev_ctl->doutput_port = PISOP16R16_Digital_Output;
                dev_ctl->dinput_port = PISOP16R16_Digital_Input;
		dev_ctl->card_id = PISOP16R16_Card_ID;
	}

        return PIODA_NOERROR;
}

/* get ixpio device module name */
WORD PIODA_GetDeviceModuleName(int fd)
{
        FILE *fd_proc;
        char rbuf[254], str[32];
        char device[254];
        char *content;
        int index = 0, cmp, card_num;
	char *delim = " ";

	card_num = find_card_num(fd);

	sprintf(str, "dev: ixpio%d",card_num);

	if(card_num > 9)
		cmp = 12;
	else
		cmp = 11;

        fd_proc = fopen(IXPIO_PROC_FILE, "r");

        if (!fd_proc)
                return FAILURE;

        while (fgets(rbuf, 254, fd_proc))
        {
                if (!strncmp(rbuf, str, cmp))
                {
			content = strtok(strtok(rbuf, "\n"),delim);

			while (content != NULL)
			{
				content = strtok (NULL, delim);
				index++;

				if(index == 4)
					strcpy(device, content);
			}
			break;
		}
        }
        fclose(fd_proc);

        if(PIODA_Set_Device_Module(device, fd))
                return FAILURE;
        return PIODA_NOERROR;
}

/* set global "device_module" */
WORD PIODA_Set_Device_Module(char dev[], int fd)
{
	dev_control_t *dev_ctl;

        dev_ctl = find_fd(fd);

	if(!strcmp(dev, "PIO-D168"))
		strcpy(dev_ctl->device_module, "ixpiod168");
	else if(!strcmp(dev, "PIO-D144"))
                strcpy(dev_ctl->device_module, "ixpiod144");
	else if(!strcmp(dev, "PIO-D56_D24"))
                strcpy(dev_ctl->device_module, "ixpiod56");
	else if(!strcmp(dev, "PIO-D24"))
                strcpy(dev_ctl->device_module, "ixpiod24");
	else if(!strcmp(dev, "PIO-D48_PEX-D48"))
		strcpy(dev_ctl->device_module, "ixpiod48_pexd48");
	else if(!strcmp(dev, "PIO-D96"))
                strcpy(dev_ctl->device_module, "ixpiod96");
	else if(!strcmp(dev, "PISO-P8R8_P8SSR8AC_P8SSR8DC"))
                strcpy(dev_ctl->device_module, "ixpisop8r8");
	else if(!strcmp(dev, "PISO-725"))
                strcpy(dev_ctl->device_module, "ixpiso725");
	else if(!strcmp(dev, "PIO-DA16_DA8_DA4"))
                strcpy(dev_ctl->device_module, "ixpioda16");
	else if(!strcmp(dev, "PISO-A64"))
                strcpy(dev_ctl->device_module, "ixpisoa64");
	else if(!strcmp(dev, "PISO-C64"))
                strcpy(dev_ctl->device_module, "ixpisoc64");
	else if(!strcmp(dev, "PISO-P64"))
                strcpy(dev_ctl->device_module, "ixpisop64");
	else if(!strcmp(dev, "PISO-813"))
                strcpy(dev_ctl->device_module, "ixpiso813");
	else if(!strcmp(dev, "PISO-P32C32"))
                strcpy(dev_ctl->device_module, "ixpisop32c32");
	else if(!strcmp(dev, "PISO-P32A32"))
                strcpy(dev_ctl->device_module, "ixpisop32a32");
	else if(!strcmp(dev, "PISO-P16R16_PEX-P8R8_P16R16"))
                strcpy(dev_ctl->device_module, "ixpisop16r16u");
	else
	{
		printf("The I/O card isn't supported yet.\n");
		strcpy(dev_ctl->device_module, "");
		return FAILURE;
	}

	//printf("%s ... Device : %s --> Module : %s\n", __FUNCTION__, dev, dev_ctl->device_module);
	return PIODA_NOERROR;
}

WORD PIODA_ReadEEPROMAll(WORD fd)
{
        WORD i;
        WORD bHi, bLo;
        WORD wTmp;
        WORD wCh;

        ixpio_reg_t reg;

        /* AUX 4/3/2 are DO, others DI */
        reg.value = 0x1c;
        reg.id = IXPIO_ACR;

        if (PIODA_OutputByte(fd, &reg))
                return PIODA_AUX_CONTROL_ERROR;

        /* ALL DO are low */
        reg.value = 0x00;
        reg.id = IXPIO_ADR;

        if (PIODA_OutputByte(fd, &reg))
                return PIODA_AUX_DATA_ERROR;

	//Read calibration data from EEPROM
	
	for(i=0; i<64; i++)
        {
                if(PIODA_EEP_READ(fd,(WORD)(i),&bHi,&bLo))
                        return PIODA_READ_EEPROM_ERROR;

                if( i<16 )
                {
                        PIODA_wN10V[fd][i]=(WORD)(bHi*256 + bLo);
                }
                else if( (i>=16) && (i<32) )
                {
                        PIODA_wP10V[fd][i-16]=(WORD)(bHi*256 + bLo);
                }
                else if( (i>=32) && (i<48) )
                {
                        PIODA_w00mA[fd][i-32]=(WORD)(bHi*256 + bLo);
                }
                else if( i>=48 )
                {
                        PIODA_w20mA[fd][i-48]=(WORD)(bHi*256 + bLo);
                }
        }

	for(i=0; i<16; i++)
        {
                wTmp = (WORD)((float)PIODA_wP10V[fd][i]-(float)PIODA_wN10V[fd][i]);

		//printf("N10V %d : %u\n", i, PIODA_wN10V[fd][i]);
                //printf("P10V %d : %u\n", i, PIODA_wP10V[fd][i]);
                //printf("Tmp V %d : %u\n", i, wTmp);

		if ( wTmp > 0 )
                {
                        PIODA_fDeltaV[fd][i]=(float)(20.0/(float)wTmp);
                }
                else
                {
                        PIODA_fDeltaV[fd][i]=0;
                }

		//printf("DV %d : %f\n", i, PIODA_fDeltaV[fd][i]);
                //printf("\n");
		wTmp = (WORD)((float)PIODA_w20mA[fd][i]-(float)PIODA_w00mA[fd][i]);

                if ( wTmp > 0 )
                {
                        PIODA_fDeltaI[fd][i]=(float)(20.0/(float)wTmp);
                }
                else
                {
                        PIODA_fDeltaI[fd][i]=0;
                }

		//printf("W00MA %d : %u\n", i, PIODA_w00mA[fd][i]);
                //printf("W20MA %d : %u\n", i, PIODA_w20mA[fd][i]);
                //printf("wTmp I %d : %u\n", i, wTmp);
                //printf("DI %d : %f\n", i, PIODA_fDeltaI[fd][i]);
                //printf("\n");

	}

        return PIODA_NOERROR;
}

WORD PIODA_EEP_READ(WORD fd, WORD wOffset, WORD *bHi, WORD *bLo)
{
        WORD wBase, wHi, wLo;
        ixpio_reg_t reg;

        if (PIODA_COMM(fd))
                return FAILURE;

	//read command & address
	
	wOffset=(WORD)(wOffset|0x80);
        if(PIODA_WR_BYTE(fd,wOffset))
                return FAILURE;

        PIODA_EEP=(WORD)(PIODA_EEP&0x0c);
        reg.id = IXPIO_ADR;
        reg.value = PIODA_EEP;

        if (PIODA_OutputByte(fd, &reg))
                return FAILURE;

        PIODA_EEP=(WORD)(PIODA_EEP&0x14);
        reg.value = PIODA_EEP;

        if (PIODA_OutputByte(fd, &reg))
                return FAILURE;

        PIODA_EEP=(WORD)(PIODA_EEP|0x08);
        reg.value = PIODA_EEP;

        if (PIODA_OutputByte(fd, &reg))
                return FAILURE;

        PIODA_EEP=(WORD)(PIODA_EEP&0x14);
        reg.value = PIODA_EEP;

        if (PIODA_OutputByte(fd, &reg))
                return FAILURE;

	PIODA_RD_BYTE(fd, &wHi);
        *bHi = (byte) wHi;
        //read low  8-bit data
        PIODA_RD_BYTE(fd, &wLo);
        *bLo = (byte) wLo;

        PIODA_EEP=(WORD)(PIODA_EEP&0x14);
        reg.value = PIODA_EEP;

        if (PIODA_OutputByte(fd, &reg))
                return FAILURE;

        PIODA_EEP=(WORD)(PIODA_EEP&0x18);
        reg.value = PIODA_EEP;

        if (PIODA_OutputByte(fd, &reg))
                return FAILURE;

        return PIODA_NOERROR;
}

// send start 2-bit (0,1)
//-----------------------------------------------
// AUX2=D/O=CS of EEP          AUX3=D/O=SK of EEP
// AUX4=D/O=DI of EEP          AUX5=D/I=DO of EEP
//-----------------------------------------------
WORD PIODA_COMM(WORD fd)
{
        ixpio_reg_t reg;

        PIODA_EEP=0x04;
        reg.value = PIODA_EEP;
        reg.id = IXPIO_ADR;

        if (PIODA_OutputByte(fd, &reg))
                return FAILURE;

        PIODA_EEP=0x0c;
        reg.value = PIODA_EEP;

        if (PIODA_OutputByte(fd, &reg))
                return FAILURE;

        PIODA_EEP=0x04;
        reg.value = PIODA_EEP;

        if (PIODA_OutputByte(fd, &reg))
                return FAILURE;

        PIODA_EEP=0x14;
        reg.value = PIODA_EEP;

        if (PIODA_OutputByte(fd, &reg))
                return FAILURE;

	
        PIODA_EEP=0x1c;
        reg.value = PIODA_EEP;

        if (PIODA_OutputByte(fd, &reg))
                return FAILURE;

        return PIODA_NOERROR;
}

WORD PIODA_WR_BYTE(WORD fd, WORD bData)
{
        WORD i,j;
        ixpio_reg_t reg;

        for(i=0; i<8; i++)
        {
                PIODA_EEP=(WORD)(PIODA_EEP&0x14);
                reg.value = PIODA_EEP;
                reg.id = IXPIO_ADR;

                if (PIODA_OutputByte(fd, &reg))
                        return FAILURE;

                j=(WORD)(bData&0x80);
                bData=(WORD)(bData<<1);

                if( j==0 )         // DI=0
                {
                        PIODA_EEP=(WORD)(PIODA_EEP&0x0c);
                        reg.value = PIODA_EEP;
                        if (PIODA_OutputByte(fd, &reg))
                                return FAILURE;
                }
		else               // DI=1
                {
                        PIODA_EEP=(WORD)(PIODA_EEP|0x10);
                        reg.value = PIODA_EEP;

                        if (PIODA_OutputByte(fd, &reg))
                                return FAILURE;
                }

                PIODA_EEP=(WORD)(PIODA_EEP|0x08);
                reg.value = PIODA_EEP;

                if (PIODA_OutputByte(fd, &reg))
                        return FAILURE;
        }

        return PIODA_NOERROR;
}

//read the next 8-bit
WORD PIODA_RD_BYTE(WORD fd, WORD *bData)
{
        byte i,j,k,Val;
        ixpio_reg_t reg, reg2;
        Val=0x80;
        k=0;

        for(i=0; i<8; i++)
        {
                PIODA_EEP=(WORD)(PIODA_EEP&0x14);
                reg.value = PIODA_EEP;
                reg.id = IXPIO_ADR;

                if (PIODA_OutputByte(fd, &reg))
                        return FAILURE;

                reg2.id = IXPIO_APSR;

                if (PIODA_InputByte(fd, &reg2))
                        return FAILURE;

                j=(byte)((reg2.value)&0x20);
                if( j!=0 ) k+=Val;     // DO=1

		PIODA_EEP=(WORD)(PIODA_EEP|0x08);
                reg.value = PIODA_EEP;

                if (PIODA_OutputByte(fd, &reg))
                        return FAILURE;

                Val=(byte)(Val>>1);
        }

        *bData = k;

        return PIODA_NOERROR;
}

WORD PIODA_8254CW(WORD fd, WORD bcd, WORD mode, WORD rl, WORD sc)
{
	WORD rs;
	dev_control_t *dev_ctl;

	dev_ctl = find_fd(fd);

	//check arg range
	if(bcd < 0 || bcd > 1 || mode < 0 || mode > 5 || rl < 0 || rl > 3 || sc < 0 || sc > 3)
		return PIODA_LIBRARY_PARAMETER_ERROR;

	rs = dev_ctl->C8254W(fd, bcd, mode, rl, sc);
	if (rs)
		return FAILURE;;

	return PIODA_NOERROR;
}

WORD PIODA_8254C0(WORD fd, WORD hbyte, WORD lbyte)
{
	WORD rs;
	dev_control_t *dev_ctl;

	dev_ctl = find_fd(fd);

	rs = dev_ctl->C8254_0(fd, hbyte, lbyte);
	if(rs)
		return FAILURE;;

	return PIODA_NOERROR;
}

WORD PIODA_8254C1(WORD fd, WORD hbyte, WORD lbyte)
{
	WORD rs;
	dev_control_t *dev_ctl;

	dev_ctl = find_fd(fd);

	rs = dev_ctl->C8254_1(fd, hbyte, lbyte);
	if(rs)
		return FAILURE;;

	return PIODA_NOERROR;
}

WORD PIODA_8254C2(WORD fd, WORD hbyte, WORD lbyte)
{
	WORD rs;
	dev_control_t *dev_ctl;

	dev_ctl = find_fd(fd);

	if(hbyte == 0 && lbyte == 0)
		return PIODA_LIBRARY_PARAMETER_ERROR;

	rs = dev_ctl->C8254_2(fd, hbyte, lbyte);
	if(rs)
		return FAILURE;

	return PIODA_NOERROR;
}

WORD PIODA_CARD_ID(WORD fd, byte *id)
{
	WORD rs;
	dev_control_t *dev_ctl;

	dev_ctl = find_fd(fd);

	rs = dev_ctl->card_id(fd, id);
	if(rs)
		return FAILURE;

	return PIODA_NOERROR;
}
